@@ -1,6 +1,6 @@
 /*
  * ModSecurity, http://www.modsecurity.org/
- * Copyright (c) 2015 Trustwave Holdings, Inc. (http://www.trustwave.com/)
+ * Copyright (c) 2015 - 2021 Trustwave Holdings, Inc. (http://www.trustwave.com/)
  *
  * You may not use this file except in compliance with
  * the License.  You may obtain a copy of the License at
@@ -29,13 +29,13 @@ namespace ctl {
 
 class RuleRemoveTargetById : public Action {
  public:
-    explicit RuleRemoveTargetById(std::string action)
+    explicit RuleRemoveTargetById(const std::string &action) 
         : Action(action, RunTimeOnlyIfMatchKind),
         m_id(0),
         m_target("") { }
 
     bool init(std::string *error) override;
-    bool evaluate(Rule *rule, Transaction *transaction) override;
+    bool evaluate(RuleWithActions *rule, Transaction *transaction) override;
 
     int m_id;
     std::string m_target;
