@@ -1,6 +1,6 @@
 /*
  * ModSecurity, http://www.modsecurity.org/
- * Copyright (c) 2015 Trustwave Holdings, Inc. (http://www.trustwave.com/)
+ * Copyright (c) 2015 - 2021 Trustwave Holdings, Inc. (http://www.trustwave.com/)
  *
  * You may not use this file except in compliance with
  * the License.  You may obtain a copy of the License at
@@ -29,11 +29,11 @@ namespace ctl {
 
 class RuleRemoveTargetByTag : public Action {
  public:
-    explicit RuleRemoveTargetByTag(std::string action)
+    explicit RuleRemoveTargetByTag(const std::string &action) 
         : Action(action, RunTimeOnlyIfMatchKind) { }
 
     bool init(std::string *error) override;
-    bool evaluate(Rule *rule, Transaction *transaction) override;
+    bool evaluate(RuleWithActions *rule, Transaction *transaction) override;
 
     std::string m_tag;
     std::string m_target;
