@@ -1,6 +1,6 @@
 /*
  * ModSecurity, http://www.modsecurity.org/
- * Copyright (c) 2015 Trustwave Holdings, Inc. (http://www.trustwave.com/)
+ * Copyright (c) 2015 - 2021 Trustwave Holdings, Inc. (http://www.trustwave.com/)
  *
  * You may not use this file except in compliance with
  * the License.  You may obtain a copy of the License at
@@ -31,12 +31,12 @@ namespace modsecurity {
 namespace actions {
 namespace transformations {
 
-HexEncode::HexEncode(std::string action)
+HexEncode::HexEncode(const std::string &action) 
     : Transformation(action) {
     this->action_kind = 1;
 }
 
-std::string HexEncode::evaluate(std::string value,
+std::string HexEncode::evaluate(const std::string &value,
     Transaction *transaction) {
 
     std::stringstream result;
