@@ -1,6 +1,6 @@
 /*
  * ModSecurity, http://www.modsecurity.org/
- * Copyright (c) 2015 Trustwave Holdings, Inc. (http://www.trustwave.com/)
+ * Copyright (c) 2015 - 2021 Trustwave Holdings, Inc. (http://www.trustwave.com/)
  *
  * You may not use this file except in compliance with
  * the License.  You may obtain a copy of the License at
@@ -30,12 +30,12 @@ namespace modsecurity {
 namespace actions {
 namespace transformations {
 
-Length::Length(std::string action)
+Length::Length(const std::string &action) 
     : Transformation(action) {
     this->action_kind = 1;
 }
 
-std::string Length::evaluate(std::string value,
+std::string Length::evaluate(const std::string &value,
     Transaction *transaction) {
 
     return std::to_string(value.size());
