@@ -138,7 +138,7 @@ bool Operator::evaluate(Transaction *transaction, const std::string& a) {
     return true;
 }
 
-Operator *Operator::instantiate(std::string op, std::string param_str) {
+Operator *Operator::instantiate(const std::string& op, const std::string& param_str) {
     std::string op_ = utils::string::tolower(op);
     std::unique_ptr<RunTimeString> param(new RunTimeString());
     param->appendText(param_str);
