@@ -1,6 +1,6 @@
 /*
  * ModSecurity, http://www.modsecurity.org/
- * Copyright (c) 2015 Trustwave Holdings, Inc. (http://www.trustwave.com/)
+ * Copyright (c) 2015 - 2021 Trustwave Holdings, Inc. (http://www.trustwave.com/)
  *
  * You may not use this file except in compliance with
  * the License.  You may obtain a copy of the License at
@@ -14,6 +14,8 @@
  */
 
 #include <string>
+#include <memory>
+#include <utility>
 
 #include "modsecurity/transaction.h"
 #include "src/operators/operator.h"
@@ -29,10 +31,8 @@ namespace operators {
 class StrEq : public Operator {
  public:
     /** @ingroup ModSecurity_Operator */
-    StrEq(std::string op, std::string param, bool negation)
-        : Operator(op, param, negation) { }
-    explicit StrEq(std::string param)
-        : Operator("StrEq", param) { }
+    explicit StrEq(std::unique_ptr<RunTimeString> param)
+        : Operator("StrEq", std::move(param)) { }
 
     bool evaluate(Transaction *transaction, const std::string &str) override;
 };
