@@ -80,6 +80,10 @@ RuleWithActions::RuleWithActions(
     m_containsStaticBlockAction(false),
     m_isChained(false) {
 
+    if (transformations != NULL) {
+        delete transformations;
+    }
+
     if (actions) {
         for (Action *a : *actions) {
             if (a->action_kind == Action::ConfigurationKind) {
@@ -215,6 +219,18 @@ void RuleWithActions::executeActionsIndependentOfChainedRuleResult(Transaction *
         }
     }
 
+    if (m_containsMultiMatchAction && !m_isChained) {
+        if (m_severity) {
+            m_severity->evaluate(this, trans, ruleMessage);
+        }
+        if (m_logData) {
+            m_logData->evaluate(this, trans, ruleMessage);
+        }
+        if (m_msg) {
+            m_msg->evaluate(this, trans, ruleMessage);
+        }
+    }
+
 }
 
 
