@@ -33,8 +33,8 @@ std::string find_resource(const std::string& file, const std::string& config,
     std::string *err);
 std::string get_path(const std::string& file);
 std::list<std::string> expandEnv(const std::string& var, int flags);
-bool createDir(std::string dir, int mode, std::string *error);
-bool isFile(std::string f);
+bool createDir(const std::string& dir, int mode, std::string *error);
+bool isFile(const std::string& f);
 
 }  // namespace utils
 }  // namespace modsecurity
